#include <stdio.h>
#include <math.h>
#include <cufft.h>
#define NX 512
#define NY 512

// 複素数の構造体定義
struct Complex {
    float real;
    float imag;
};

int main() {
    const int size = NX * NY;
    const int complexSize = size * sizeof(Complex);

    cufftHandle plan;

    // ホスト上の入力データと出力データ
    Complex hostInput[NX][NY];
    Complex hostOutput[size];

    // デバイス上の入力データと出力データ
    Complex *deviceInput;
    Complex *deviceOutput;

    for (int i = 0; i < NX; i++) {
        for (int j = 0; j < NY; j++) {
            hostInput[i][j].real = static_cast<float>(sin(i + j));
            hostInput[i][j].imag = static_cast<float>(cos(i + j));
        }
    }

    // デバイスメモリを確保
    cudaMalloc((void**)&deviceInput, complexSize);
    cudaMalloc((void**)&deviceOutput, complexSize);

    // 入力データ転送
    cudaMemcpy(deviceInput, hostInput, complexSize, cudaMemcpyHostToDevice);

    // 2次元FFTのプランを作成
    cufftPlan2d(&plan, NX, NY, CUFFT_C2C);

    // FFTを実行
    cufftExecC2C(plan, (cufftComplex*)deviceInput, (cufftComplex*)deviceOutput, CUFFT_FORWARD);

    // 出力データ転送
    cudaMemcpy(hostOutput, deviceOutput, complexSize, cudaMemcpyDeviceToHost);

    // 結果の出力
    for (int i = 0; i < NX; i++) {
        for (int j = 0; j < NY; j++) {
            int idx = i * NY + j;
            printf("Input: %f + %fI, Output: %f + %fI\n", hostInput[i][j].real, hostInput[i][j].imag, hostOutput[idx].real, hostOutput[idx].imag);
        }
    }

    cufftDestroy(plan);
    cudaFree(deviceInput);
    cudaFree(deviceOutput);

    return 0;
}